#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "StartupLocationSelection.h"
#include "StartupLocationMapObject.h"
#include "StartupLocationHeadingHandle.h"
#include "StartupLocationRadiusHandle.h"
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>
#include <Math/TDRectangle.h>

IMPLEMENT_CLASS(StartupLocationSelection,AINetworkVertexSelection);

StartupLocationSelection::StartupLocationSelection(MapSelectionManager* manager, StartupLocationMapObject* content):
        AINetworkVertexSelection(manager,content) {
}

StartupLocationSelection::~StartupLocationSelection() {
}

MapSelectionHandle* StartupLocationSelection::HitHandle(Point3D p) {
        if (!HandlesActive())
                return (MapSelectionHandle*)NULL;
        
        /* the tip handle */
        Point3D corners[4];
        GetContent()->GetLocalCornerPoints(GetSelectionManager()->GetEditor(),corners);
        Point3D boxSize=GetHandleBoxLogicalSize();
        if (corners[1].distance3Dsquared(p)<boxSize.x()*boxSize.x()) {
                return new StartupLocationHeadingHandle(this);
        }
        
        /* the radius handle */
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        MapProjection *proj=editor->GetProjection();
        Point3D center=GetContent()->GetPosition();
        double radius=GetContent()->GetRadius();
        Point3D localcenter=proj->ConvertToLocal(center);
        double dist=localcenter.distance3D(p);
        if (2.0*fabs(dist-radius)<boxSize.x()) {
                return new StartupLocationRadiusHandle(this);
        }
        return AINetworkVertexSelection::HitHandle(p);
}

wxRect StartupLocationSelection::GetDeviceBounds() const {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        wxRect r=GetContent()->GetDeviceBounds(editor);
        if (HandlesActive()) {
                MapCoordinateSystem* cs=&editor->GetCoordinateSystem();
                MapProjection* proj=editor->GetProjection();
                
                Point3D center=proj->ConvertToLocal(GetContent()->GetPosition());
                wxPoint devicecenter=cs->LogicalToDevice(center);
                wxCoord deviceradius=cs->LogicalToDeviceRelX(GetContent()->GetRadius());
                wxRect radiusRect=wxRect(devicecenter.x-deviceradius,
                                         devicecenter.y-deviceradius,
                                         2*deviceradius,
                                         2*deviceradius);
                r=r.Union(radiusRect);
        }
        wxSize handleSize=GetHandleBoxDeviceSize();
        r.Inflate(handleSize.x,handleSize.y);
        return r;
}

void StartupLocationSelection::Paint(wxDC& dc) {
        if (HandlesActive()) {
                PaintHandles(dc);
        }
        AINetworkVertexSelection::Paint(dc);
}

void StartupLocationSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        AINetworkVertexSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(StartupLocationSelection)))
                return;
        
        propgrid->Append(wxStringProperty(wxT("Name"), wxT("StartupLocationSelection_Name")));
        propgrid->Append(wxStringProperty(wxT("Gate Name"), wxT("StartupLocationSelection_GateName")));
        propgrid->Append(wxStringProperty(wxT("Gate Number"), wxT("StartupLocationSelection_GateNumber")));

        wxPGChoices gateTypeChoices;
        gateTypeChoices.Add(wxT("Gate"),StartupLocationMapObject::GateType_Gate);
        gateTypeChoices.Add(wxT("Cargo"),StartupLocationMapObject::GateType_Cargo);
        gateTypeChoices.Add(wxT("General Aviation"),StartupLocationMapObject::GateType_GA);
        gateTypeChoices.Add(wxT("Military (Fighter)"),StartupLocationMapObject::GateType_Mil_Fighter);
        gateTypeChoices.Add(wxT("Military (Cargo)"),StartupLocationMapObject::GateType_Mil_Cargo);
        
        propgrid->Append(wxEnumProperty(wxT("Gate Type"),
                                        wxT("StartupLocationSelection_GateType"),
                                        gateTypeChoices));
        
        propgrid->Append(wxStringProperty(wxT("Airline Codes"), wxT("StartupLocationSelection_AirlineCodes")));
        propgrid->Append(wxFloatProperty(wxT("Heading"), wxT("StartupLocationSelection_Heading")));
        propgrid->Append(wxFloatProperty(wxT("Radius"), wxT("StartupLocationSelection_Radius")));

        propgrid->Disable(wxT("StartupLocationSelection_Name"));
}

void StartupLocationSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        AINetworkVertexSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(StartupLocationSelection)))
                return;

        Point3D center=GetContent()->GetPosition();
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_Name"),GetContent()->GetName());
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_GateName"),GetContent()->GetGateName());
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_GateNumber"),GetContent()->GetGateNumber());
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_GateType"),GetContent()->GetGateType());
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_AirlineCodes"),GetContent()->GetAirlineCodes());
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_Heading"),GetContent()->GetHeading());
        propgrid->SetPropertyValue(wxT("StartupLocationSelection_Radius"),GetContent()->GetRadius());
}

void StartupLocationSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        AINetworkVertexSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(StartupLocationSelection)))
                return;
        StartupLocationMapObject* firstContent=wxDynamicCast(firstSelection,StartupLocationSelection)->GetContent();
        StartupLocationMapObject* myContent=GetContent();

        if (firstContent->GetName().Cmp(myContent->GetName())!=0)
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_Name"));
        if (firstContent->GetGateName().Cmp(myContent->GetGateName())!=0)
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_GateName"));
        if (firstContent->GetGateNumber().Cmp(myContent->GetGateNumber())!=0)
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_GateNumber"));
        if (myContent->GetGateType()!=firstContent->GetGateType())
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_GateType"));
        if (firstContent->GetAirlineCodes().Cmp(myContent->GetAirlineCodes())!=0)
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_AirlineCodes"));
        if (firstContent->GetHeading()!=myContent->GetHeading())
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_Heading"));
        if (firstContent->GetRadius()!=myContent->GetRadius())
                propgrid->SetPropertyUnspecified(wxT("StartupLocationSelection_Radius"));
}

void StartupLocationSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("StartupLocationSelection_GateName"))==0) {
                GetContent()->SetGateName(property->GetValueAsString());
        } else if (name.Cmp(wxT("StartupLocationSelection_GateNumber"))==0) {
                GetContent()->SetGateNumber(property->GetValueAsString());
        } else if (name.Cmp(wxT("StartupLocationSelection_GateType"))==0) {
                GetContent()->SetGateType((StartupLocationMapObject::GateType)value.GetLong());
        } else if (name.Cmp(wxT("StartupLocationSelection_AirlineCodes"))==0) {
                GetContent()->SetAirlineCodes(property->GetValueAsString());
        } else if (name.Cmp(wxT("StartupLocationSelection_Heading"))==0) {
                GetContent()->SetHeading(value.GetDouble());
        } else if (name.Cmp(wxT("StartupLocationSelection_Radius"))==0) {
                GetContent()->SetRadius(value.GetDouble());
        } else {
                AINetworkVertexSelection::UpdateObjectFromProperty(property);
        }
}

void StartupLocationSelection::PaintHandles(wxDC& dc) {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapCoordinateSystem* cs=&editor->GetCoordinateSystem();
        
        wxSize rectsize=GetHandleBoxDeviceSize();

        wxPen handlePen=wxPen(Selection_Handle_Colour);

        dc.SetPen(handlePen);
        dc.SetBrush(*wxTRANSPARENT_BRUSH);

        /* draw tip handle */
        Point3D corners[4];
        GetContent()->GetLocalCornerPoints(GetSelectionManager()->GetEditor(),corners);
        wxPoint devicetip=cs->LogicalToDevice(corners[1]);
        dc.DrawCircle(devicetip,rectsize.x/2);
        
        /* draw radius handle */
        MapProjection* proj=editor->GetProjection();
        Point3D center=proj->ConvertToLocal(GetContent()->GetPosition());
        wxPoint devicecenter=cs->LogicalToDevice(center);
        wxCoord deviceradius=cs->LogicalToDeviceRelX(GetContent()->GetRadius());
        wxRect radiusRect=wxRect(devicecenter.x-deviceradius,
                                 devicecenter.y-deviceradius,
                                 2*deviceradius,
                                 2*deviceradius);

        dc.DrawCircle(devicecenter,deviceradius);

        dc.SetBrush(wxNullBrush);
        dc.SetPen(wxNullPen);
}

