#include "MapSimpleBackgroundImage.h"

#include "MapSimpleBackgroundImageSelection.h"

#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>

IMPLEMENT_CLASS(MapSimpleBackgroundImage,MapObject)

MapSimpleBackgroundImage::MapSimpleBackgroundImage(TransactionManager* transactionManager,
                                                   Point3D northWestPos,
                                                   double scale):
        MapObject(transactionManager)
{
        TRANSACTIONOBJECT_STATEVAR(_northWestPos)=northWestPos;
        TRANSACTIONOBJECT_STATEVAR(_scale)=scale;
}

MapSimpleBackgroundImage::~MapSimpleBackgroundImage() {
}

void MapSimpleBackgroundImage::Paint(wxDC& dc, MapEditor* editor, const MapPass& pass) {
        if (pass!=StandardPass)
                return;
        
        const wxImage& theImage=GetImage();
        if (!theImage.Ok())
                return;
        wxRect imgRect=GetDeviceBounds(editor);
        
        wxCoord clipx,clipy,clipw,cliph;
        dc.GetClippingBox(&clipx,&clipy,&clipw,&cliph);
        wxRect clipRect=wxRect(clipx,clipy,clipw,cliph);
        
        if (!clipRect.Intersects(imgRect))
                return;
        
        /*
         * We need to do some drawing optimisation.
         * In the typical case - e.g. when zooming in really far - we don't need
         * to scale and draw all of the image. In fact, we would be drawing a
         * whole lot of the image which isn't visible at all.
         * So we find out which part of the image is actually drawn and
         * scale and draw only that part.
         * Trust me, this makes up for a much better performance.
         */
        wxRect drawRect=clipRect.Intersect(imgRect);

        wxRect subImgRect;
        float mx,cx;
        float my,cy;
        
        mx=(float)theImage.GetWidth()/(float)imgRect.GetWidth();
        my=(float)theImage.GetHeight()/(float)imgRect.GetHeight();
        cx=-imgRect.GetLeft()*mx;
        cy=-imgRect.GetTop()*my;
        subImgRect.SetX((wxCoord)::floor(drawRect.GetX()*mx+cx));
        subImgRect.SetY((wxCoord)::floor(drawRect.GetY()*my+cy));
        subImgRect.SetWidth((wxCoord)::ceil(drawRect.GetWidth()*mx));
        subImgRect.SetHeight((wxCoord)::ceil(drawRect.GetHeight()*my));
        
        /* This may happen due to rounding error */
        if (subImgRect.GetLeft()<0) {
                subImgRect.SetX(0);
        }
        if (subImgRect.GetTop()<0) {
                subImgRect.SetY(0);
        }
        if (subImgRect.GetRight()>=theImage.GetWidth()) {
                subImgRect.SetWidth(theImage.GetWidth()-subImgRect.GetX());
        }
        if (subImgRect.GetBottom()>=theImage.GetHeight()) {
                subImgRect.SetHeight(theImage.GetHeight()-subImgRect.GetY());
        }
        
        if (subImgRect.IsEmpty())
                return;
        
        wxImage img=theImage.GetSubImage(subImgRect);
        img=img.Scale(drawRect.GetWidth(),drawRect.GetHeight());
        
        wxBitmap bmp(img);
        dc.DrawBitmap(bmp,drawRect.GetLeft(),drawRect.GetTop(),true);
}

bool MapSimpleBackgroundImage::Hit(MapEditor* editor, TDRectangle r) {
        return GetLocalBounds(editor).Intersects(r);
}

MapSelection* MapSimpleBackgroundImage::CreateSelection(MapSelectionManager* manager) {
        return new MapSimpleBackgroundImageSelection(manager,this);
}

wxRect MapSimpleBackgroundImage::GetDeviceBounds(MapEditor* editor) {
        const wxImage& theImage=GetImage();
        if (!theImage.Ok()) {
                return wxRect();
        }
        MapProjection *proj=editor->GetProjection();
        MapCoordinateSystem *cs=&editor->GetCoordinateSystem();
        
        wxPoint imageOffset=GetImageOffset();
        
        wxRect devImgRect;
        Point3D localPos;
        wxPoint devPos;
        wxCoord devOffx,devOffy;
        int devWidth,devHeight;
        
        // Determine the device bounding rectangle of the image
        localPos=proj->ConvertToLocal(GetNorthWestPos());
        devPos=cs->LogicalToDevice(localPos);
        devOffx=cs->LogicalToDeviceRelX(imageOffset.x*GetScale());
        devOffy=cs->LogicalToDeviceRelY(-imageOffset.y*GetScale());
        devWidth=abs(cs->LogicalToDeviceRelX(theImage.GetWidth()*GetScale()));
        devHeight=abs(cs->LogicalToDeviceRelY(theImage.GetHeight()*GetScale()));
        devImgRect=wxRect(devPos.x+devOffx,devPos.y+devOffy,
                                 devWidth,devHeight);
        
        return devImgRect;
}

TDRectangle MapSimpleBackgroundImage::GetGeographicBounds() {
        /* We can't give geographic bounds for an image without a projection */
        return TDRectangle();
}

const wxImage& MapSimpleBackgroundImage::LoadImage() {
        if (!::wxFileExists(TRANSACTIONOBJECT_STATEVAR(_fileName))) {
                _image.Destroy();
                return _image;
        }
        Damage();
        TriggerBeforeObjectChange();
        _image.LoadFile(TRANSACTIONOBJECT_STATEVAR(_fileName));
        TriggerAfterObjectChange();
        Damage();
        return _image;
}

void MapSimpleBackgroundImage::GetLocalCorners(MapEditor* editor, Point3D corners[4]) {
        MapProjection* proj=editor->GetProjection();
        Point3D localNWPos=proj->ConvertToLocal(GetNorthWestPos());
        wxCoord width=0,height=0;
        
        if (_image.Ok()) {
                width=_image.GetWidth();
                height=_image.GetHeight();
        }
        double scale=GetScale();
        Point3D vx,vy;
        vx=Point3D(scale,0.0,0.0);
        vy=Point3D(0.0,-scale,0.0);
        corners[0]=localNWPos;
        corners[1]=localNWPos+width*vx;
        corners[2]=localNWPos+width*vx+height*vy;
        corners[3]=localNWPos+height*vy;
}

void MapSimpleBackgroundImage::GetDeviceCorners(MapEditor* editor, wxPoint corners[4]) {
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        Point3D localCorners[4];
        GetLocalCorners(editor,localCorners);
        for (unsigned int i=0;i<4;i++) {
                corners[i]=cs.LogicalToDevice(localCorners[i]);
        }
}

