#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapSelection.h"
#include "MapObject.h"
#include "MapEditor.h"
#include "MapSelectionActionsHandler.h"
#include "MapSelectionManager.h"
#include "MapSelectionHandle.h"
#include "MapObjectContainer.h"

#include <wx/listimpl.cpp>

WX_DEFINE_LIST(MapSelectionList);

IMPLEMENT_ABSTRACT_CLASS(MapSelection,wxObject);

wxColour MapSelection::Selection_Handle_Colour=wxColour(255,0,0);
wxColour MapSelection::Selection_Marking_Colour=wxColour(0,0,0);

MapSelection::MapSelection(MapSelectionManager* manager, MapObject* content):
        _selectionManager(manager),
        _contentContainer(content->GetContainer()),
        _content(content),
        _refCount(0),
        _activationCount(0),
        _handlesActive(false) {
        wxASSERT_MSG(_content!=NULL,
                     wxT("Cannot create selection for NULL-content"));
}

MapSelection::~MapSelection() {
        wxASSERT_MSG(_activationCount==0,
                     wxT("Destroying active selection"));
}

void MapSelection::ActivateSelection() {
        if (_activationCount++>0)
                return;
	DoActivateSelection();
}

void MapSelection::DeactivateSelection() {
        wxASSERT_MSG(_activationCount>0,wxT("Deactivating inactive selection"));
        if (--_activationCount!=0)
                return;
	DoDeactivateSelection();
}

void MapSelection::DoActivateSelection() {
        _content->RegisterChangeListener(this);
}

void MapSelection::DoDeactivateSelection() {
        _content->UnregisterChangeListener(this);
}

bool MapSelection::Hit(Point3D p) {
        if (GetContent()->Hit(GetSelectionManager()->GetEditor(),
                              TDRectangle(p,p)))
                return true;
        MapSelectionHandle* handle=HitHandle(p);
        if (handle) {
                delete handle;
                return true;
        }
        return false;
}

MapSelectionHandle* MapSelection::HitHandle(Point3D p) {
        return (MapSelectionHandle*)NULL;
}

wxRect MapSelection::GetDeviceBounds() const {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        wxRect r=GetContent()->GetDeviceBounds(editor);
        wxSize handleSize=GetHandleBoxDeviceSize();
        r.Inflate(handleSize.x,handleSize.y);
        return r;
}

TDRectangle MapSelection::GetLocalBounds() const {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        wxRect devbounds=GetDeviceBounds();
        
        Point3D localTopLeft=cs.DeviceToLogical(devbounds.GetTopLeft());
        Point3D localBottomRight=cs.DeviceToLogical(devbounds.GetBottomRight());
        
        return TDRectangle(localTopLeft,localBottomRight);
}

void MapSelection::ProvideActionMenuForClass(wxMenu& popupMenu, wxClassInfo* targetClass) {
        if (!targetClass->IsKindOf(CLASSINFO(MapSelection)))
                return;
        wxMenu* orderMenu=new wxMenu();
        orderMenu->SetEventHandler(popupMenu.GetEventHandler());
        orderMenu->SetClientData(popupMenu.GetClientData());
        wxMenuItem *item;
        
        item=orderMenu->Append(wxID_ANY,wxT("Bring to Front"));
        orderMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(MapSelectionActionsHandler::OnBringToFront),
                                              NULL,
                                              MapSelectionActionsHandler::GetInstance());
        item=orderMenu->Append(wxID_ANY,wxT("Bring forward"));
        orderMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(MapSelectionActionsHandler::OnBringForward),
                                              NULL,
                                              MapSelectionActionsHandler::GetInstance());
        item=orderMenu->Append(wxID_ANY,wxT("Bring backward"));
        orderMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(MapSelectionActionsHandler::OnBringBackward),
                                              NULL,
                                              MapSelectionActionsHandler::GetInstance());
        item=orderMenu->Append(wxID_ANY,wxT("Bring to Back"));
        orderMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(MapSelectionActionsHandler::OnBringToBack),
                                              NULL,
                                              MapSelectionActionsHandler::GetInstance());
        
        popupMenu.Append(wxID_HIGHEST,wxT("Ordering"),orderMenu);
        item=popupMenu.Append(wxID_ANY,wxT("Delete"));
        popupMenu.GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(MapSelectionActionsHandler::OnDelete),
                                              NULL,
                                              MapSelectionActionsHandler::GetInstance());
}

void MapSelection::Damage() const {
        GetSelectionManager()->GetEditor()->DamageRect(GetDeviceBounds());
}

void MapSelection::Paint(wxDC& dc) {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        wxRect bounds=GetDeviceBounds();

        wxSize handleSize=GetHandleBoxDeviceSize();
        bounds.Deflate(handleSize.x,handleSize.y);
        
        wxPen markingPen=wxPen(Selection_Marking_Colour);
        wxBrush markingBrush=wxBrush(Selection_Marking_Colour);
        dc.SetPen(markingPen);
        dc.SetBrush(markingBrush);
        dc.DrawRectangle(bounds.GetLeft()   ,bounds.GetTop(),   -handleSize.x,-handleSize.y);
        dc.DrawRectangle(bounds.GetRight()+1,bounds.GetTop(),    handleSize.x,-handleSize.y);
        dc.DrawRectangle(bounds.GetLeft()   ,bounds.GetBottom(),-handleSize.x, handleSize.y);
        dc.DrawRectangle(bounds.GetRight()+1,bounds.GetBottom(), handleSize.x, handleSize.y);
        dc.SetBrush(wxNullBrush);
        dc.SetPen(wxNullPen);
}

void MapSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
}

void MapSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
}

void MapSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
}

void MapSelection::UpdateObjectFromProperty(wxPGProperty* property) {
}

void MapSelection::BeforeObjectChange(TransactionObject* object) {
        wxASSERT_MSG(object==_content,
                     wxT("Informed about change on wrong object"));
        Damage(); // mark old region as damaged
}

void MapSelection::AfterObjectChange(TransactionObject* object) {
        wxASSERT_MSG(object==_content,
                     wxT("Informed about change on wrong object"));
        Damage(); // mark new region as damaged
}

MapEditor* MapSelection::GetEditor() const {
        return GetSelectionManager()->GetEditor();
}

wxSize MapSelection::GetHandleBoxDeviceSize() const {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        wxCoord rectdx=cs.BoundedDeviceSizeX(5,15,20);
        wxCoord rectdy=cs.BoundedDeviceSizeY(5,15,20);
        return wxSize(rectdx,rectdy);
}

Point3D MapSelection::GetHandleBoxLogicalSize() const {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        double rectdx=cs.BoundedLogicalSizeX(5,15,20);
        double rectdy=cs.BoundedLogicalSizeY(5,15,20);
        return Point3D(rectdx,rectdy,0);
}

