#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapSelectionManager.h"
#include "MapObject.h"
#include "MapEditor.h"
#include <wx/listimpl.cpp>
WX_DEFINE_LIST(MapSelectionChangeListenerList);

IMPLEMENT_CLASS2(MapSelectionManager,TransactionObject,wxEvtHandler);

MapSelectionManager::MapSelectionManager(MapEditor* editor):
        TransactionObject(editor->GetTransactionManager()),
        _editor(editor) {
}

MapSelectionManager::~MapSelectionManager() {
        MapSelectionList::Node* node;
        for (node=_selections.GetFirst();node;node=node->GetNext()) {
                node->GetData()->DeactivateSelection();
                node->GetData()->UnRef();
        }
}

MapSelection* MapSelectionManager::AddSelectionFor(MapObject* obj) {
        MapSelection* sel;
        
        sel=GetSelectionForObject(obj);
        if (sel)
                return sel;
        
        sel=obj->CreateSelection(this);
        if (_selections.GetCount()==1) {
                /* there was a singleton selection, which will now become non-singleton */
                _selections.GetFirst()->GetData()->DeactivateHandles();
        }
        
        BeforeWrite();
        sel->Ref();
        _selections.Append(sel);
        _selectionMap[obj]=sel;
        sel->ActivateSelection();
        if (_selections.GetCount()==1) {
                /* the new selection will be a singleton selection */
                sel->ActivateHandles();
        }        sel->Damage();
        AfterWrite();
        
        MapSelectionList sellist;
        sellist.Append(sel);
        FireSelectionsAdded(sellist);
        
        return sel;
}

void MapSelectionManager::RemoveSelection(MapSelection* selection) {
        MapSelectionList sellist;
        sellist.Append(selection);
        FireSelectionsRemoved(sellist);
        
        BeforeWrite();
        selection->Damage();
        selection->DeactivateHandles();
        selection->DeactivateSelection();
        _selectionMap.erase(selection->GetContent());
        _selections.DeleteObject(selection);
        selection->UnRef();
        AfterWrite();
        
        if (_selections.GetCount()==1) {
                /* there is now a singleton selection */
                _selections.GetFirst()->GetData()->ActivateHandles();
        }
}

void MapSelectionManager::DeselectAll() {
        MapSelectionList::Node *node;
        if (_selections.GetCount()==0)
                return;
        BeforeWrite();

        FireSelectionsRemoved(_selections);

        while ((node=_selections.GetFirst())!=(MapSelectionList::Node*)NULL) {
                MapSelection* sel=node->GetData();
                sel->Damage();
                sel->DeactivateHandles();
                sel->DeactivateSelection();
                _selectionMap.erase(sel->GetContent());
                _selections.DeleteNode(node);
                sel->UnRef();
        }
        
        AfterWrite();
}

MapSelection* MapSelectionManager::GetSelectionForObject(MapObject* obj) {
        MapSelectionMap::iterator it;
        it=_selectionMap.find(obj);
        if (it==_selectionMap.end())
                return NULL;
        return it->second;
}

void MapSelectionManager::Paint(wxDC& dc) {
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext())
                node->GetData()->Paint(dc);
}

void MapSelectionManager::Damage() {
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext())
                node->GetData()->Damage();
}

bool MapSelectionManager::Hit(Point3D p) {
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext())
                if (node->GetData()->Hit(p))
                        return true;
        return false;
}

MapSelectionHandle* MapSelectionManager::HitHandle(Point3D p) {
        MapSelectionList::Node *node;
        MapSelectionHandle* handle;
        for (node=_selections.GetFirst();node;node=node->GetNext()) {
                handle=node->GetData()->HitHandle(p);
                if (handle)
                        return handle;
        }
        return (MapSelectionHandle*)NULL;
}

TDRectangle MapSelectionManager::GetLocalBounds() const {
        TDRectangle r;
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext())
                r=TDRectangle::Union(r,node->GetData()->GetLocalBounds());
        return r;
}

void MapSelectionManager::StartDrag(Point3D currentPosition) {
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext())
                node->GetData()->StartDrag(currentPosition);
}

void MapSelectionManager::Drag(Point3D currentPosition) {
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext())
                node->GetData()->Drag(currentPosition);
}

wxString MapSelectionManager::EndDrag() {
        if (_selections.GetCount()==1)
                return _selections.GetFirst()->GetData()->EndDrag();
        return wxT("Dragging multiple objects");
}

WX_DECLARE_STRING_HASH_MAP(int,ClassCountMap);

wxClassInfo* MapSelectionManager::FindCommonSelectionSuperClass() {
        MapSelectionList::Node *node;
        wxClassInfo *maxClass=(wxClassInfo*)NULL;
        wxClassInfo *classMapSelection=CLASSINFO(MapSelection);
        size_t maxCount=0,c;
        ClassCountMap classCount;
        
        for (node=_selections.GetFirst();node;node=node->GetNext()) {
                wxClassInfo *classInfo=node->GetData()->GetClassInfo();
                while (classInfo!=(wxClassInfo*)NULL) {
                        if (classCount.find(classInfo->GetClassName())==classCount.end())
                                c=classCount[classInfo->GetClassName()]=1;
                        else
                                c=++classCount[classInfo->GetClassName()];
                        if (c>maxCount) {
                                maxClass=classInfo;
                                maxCount=c;
                        }
                        if (classInfo==classMapSelection)
                                break;
                        classInfo=wxClassInfo::FindClass(classInfo->GetBaseClassName1());
                }
        }
        if (maxCount==_selections.GetCount())
                return maxClass;
        // else
        return classMapSelection;
}

MapSelection* MapSelectionManager::FindFirstSelectionOfType(wxClassInfo* classInfo) {
        wxASSERT_MSG(classInfo->IsKindOf(CLASSINFO(MapSelection)),
                     wxT("Can only hold selections of kind MapSelection"));
        
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext()) {
                if (node->GetData()->IsKindOf(classInfo))
                        return node->GetData();
        }
        return (MapSelection*)NULL;
}

void MapSelectionManager::GetSelectionsOfType(MapSelectionList& list, wxClassInfo* classInfo) {
        wxASSERT_MSG(classInfo->IsKindOf(CLASSINFO(MapSelection)),
                     wxT("Can only hold selections of kind MapSelection"));
        
        MapSelectionList::Node *node;
        for (node=_selections.GetFirst();node;node=node->GetNext()) {
                if (node->GetData()->IsKindOf(classInfo))
                        list.Append(node->GetData());
        }
}

void MapSelectionManager::RestoreMemento(wxObject* object) {
	TransactionObject::RestoreMemento(object);
        Memento* memento=dynamic_cast<Memento*>(object);
        wxASSERT(memento);
        MapSelectionList::Node *node;
        
        Damage();
        
        FireSelectionsRemoved(_selections);
        FireSelectionsAdded(memento->_selections);
        
        for (node=memento->_selections.GetFirst();node;node=node->GetNext()) {
                node->GetData()->ActivateSelection();
                node->GetData()->DeactivateHandles();
                node->GetData()->Ref();
        }
        for (node=_selections.GetFirst();node;node=node->GetNext()) {
                node->GetData()->DeactivateSelection();
                node->GetData()->DeactivateHandles();
                node->GetData()->UnRef();
        }
        
        _selections=memento->_selections;
        _selectionMap=memento->_selectionMap;
        
        if (_selections.GetCount()==1) {
                /* there is now a singleton selection */
                _selections.GetFirst()->GetData()->ActivateHandles();
        }
        Damage();
}

